from pydantic import BaseModel, Field


class SendSms(BaseModel):
    text: str
    shortened_link: str = Field(..., alias="cuted_link")
    balance: float
    balance_after: float = Field(..., alias="sent_before")
    message_id: int


class SendCustomSms(BaseModel):
    price: float
    balance: float
    balance_after: float = Field(..., alias="sent_before")
    message_id: int


class SmsStatus(BaseModel):
    status: bool



from pydantic import BaseModel, Field, validator, field_validator


class DomainInfo(BaseModel):
    isBanned: bool = Field(..., alias="rt_check")
    isWorking: bool = Field(..., alias="a_check")

    @field_validator('isBanned')
    def invert_isBanned(cls, v):
        return not v



from typing import List

from pydantic import BaseModel, Field


class Services(BaseModel):
    services: List["Service"]


class Service(BaseModel):
    name: str
    pattern: str
    number_pattern: str
    number_example: str
    price: float
    sid: str
    version_1: str = Field(..., alias="text_1")
    version_2: str = Field(..., alias="text_2")
    refund: str = Field(..., alias="text_3")



from datetime import datetime
import enum
from typing import List

from aiohttp import ClientSession, ContentTypeError
from .models import Countries, Services, Profile, Service, DomainInfo, SendSms, SendCustomSms, SmsStatus
from apscheduler.schedulers.asyncio import AsyncIOScheduler


class RequestMethods(enum.StrEnum):
    POST = "POST"
    GET = "GET"
    PUT = "PUT"


class MeowSMS:
    BASE_URL = "https://api.meowsms.app/"

    def __init__(self, token: str = None, proxy: str = None):
        if not token or token and not isinstance(token, str):
            raise TypeError("Token must be a string")
        self.session = ClientSession()
        self.headers = {"Authorization": f"Bearer {token}"}
        self.services: List[Service] = []
        self.patterns: List[str] = []
        self.number_patterns: List[str] = []
        self.proxy = proxy

        self.scheduler = AsyncIOScheduler()
        self.scheduler.start()

        # Schedule the update_services method to run daily
        self.scheduler.add_job(self._update_services, 'interval', days=1, next_run_time=datetime.now())

    async def _update_services(self) -> None:
        # This method fetches the services and updates self._services
        self.services = (await self._services()).services
        self.number_patterns = self.modify_regex_patterns()
        self.patterns = list(map(lambda x: x.pattern, self.services))

    async def countries(self) -> Countries:
        response = await self.__request(RequestMethods.GET, self.BASE_URL + "getCountries/")
        return Countries(**response)

    async def _services(self) -> Services:
        response: dict = await self.__request(RequestMethods.GET, self.BASE_URL + "getServices/")
        services = Services(**response)
        self.services = services.services
        self.number_patterns = self.modify_regex_patterns()
        self.patterns = list(map(lambda x: x.pattern, self.services))
        return services

    async def me(self) -> Profile:
        response = await self.__request(RequestMethods.GET, self.BASE_URL + "getMe/")
        return Profile(**response)

    # async def cutLink(self, link: str):
    #     response = await self.__request(RequestMethods.PUT, self.BASE_URL + "cutLink/", json={"link": link})
    #     print(response)

    async def checkDomain(self, link: str) -> DomainInfo:
        response = await self.__request(RequestMethods.GET, self.BASE_URL + "checkDomainStatus/", json={"link": link})
        return DomainInfo(**response)

    async def sendSms(self, number: str, service: str, link: str, template: int, webhook_url: str = None,
                      worker_id: int = None):
        payload = {"number": number, "service": service, "link": link}
        if template is not None:
            payload["template"] = template
        if webhook_url is not None:
            payload["webhook_url"] = webhook_url
        if worker_id is not None:
            payload["worker_id"] = worker_id
        response = await self.__request(RequestMethods.POST, self.BASE_URL + "sendSMS/", json=payload)
        return SendSms(**response)

    async def sendCustomSms(self, number: str, sender_name: str, text: str, webhook_url: str = None,
                            worker_id: int = None):
        payload = {"number": number, "name": sender_name, "text": text}
        if webhook_url is not None:
            payload["webhook_url"] = webhook_url
        if worker_id is not None:
            payload["worker_id"] = worker_id
        response = await self.__request(RequestMethods.POST, self.BASE_URL + "sendCustomSMS/", json=payload)
        return SendCustomSms(**response)

    async def checkStatus(self, message_id: int):
        response = await self.__request(RequestMethods.GET, self.BASE_URL + "getSMSStatus/",
                                        json={"message_id": message_id})
        return SmsStatus(**response)

    async def __request(self, method: RequestMethods, url: str, **args) -> dict:
        response = await self.session.request(method, url, proxy=self.proxy, headers=self.headers, **args)
        try:
            jsoned: dict = await response.json()
        except ContentTypeError:
            raise Exception(await response.text())
        if jsoned.get("result", None) is not None and not jsoned.pop("result"):
            error = jsoned.get("error")
            raise Exception(error)
        return jsoned

    def modify_regex_patterns(self) -> List[str]:
        return [service.number_pattern.strip('/').replace('\\\\', '\\') for service in self.services]



from typing import List

from pydantic import BaseModel


class Countries(BaseModel):
    countries: List["Country"]


class Country(BaseModel):
    name: str
    pattern: str
    number_pattern: str
    number_example: str
    price: float



from setuptools import setup, find_packages

__version__ = "0.1.0.3"

with open("README.md", "r", encoding="utf-8") as desc_long:
    description_long = desc_long.read()

setup(
    name='meow-sms',
    version=__version__,
    author='fafatypoty',
    author_email='fafatypoty@proton.me',
    description=(
        'Асинхронный модуль для работы с Meow SMS API'
    ),
    long_description=description_long,
    long_description_content_type='text/markdown',
    project_urls={
        "Telegram": "https://t.me/fafatypoty_cloaca",
        "Github": "https://github.com/fafatypoty/meow-sms",
    },
    license='Apache License, Version 2.0, see LICENSE file',
    packages=find_packages(include=["meow_sms", "meow_sms.*"]),
    zip_safe=False,
    install_requires=["setuptools",
                      "pydantic",
                      "aiohttp",
                      "APScheduler"],
    python_requires=">=3.7",
    keywords=['meow', 'meow api', 'asyncmeow', 'meow sms', 'meowsms'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package

        'Intended Audience :: Developers',  # Define that your audience are developers
        'Topic :: Software Development :: Build Tools',

        'License :: OSI Approved :: Apache Software License',  # Again, pick a license

        'Programming Language :: Python :: 3',  # Specify which pyhton versions that you want to support
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ],
)
